#!/bin/sh
#
# update.sh, Version 3
#
# (C) Copyright IBM Corp. 2005
#
# This script can be used to mount a DCSS for software updates. Requirements:
# - /sbin/xipinit.sh in with up-to-date RODIRS defined
# - superuser privileges in Linux
# - class E privileges in z/VM

#temporary device node used
TEMPNODE=/tmp/dcssupdate.dev

#uncommend following 2 lines for debugging only
#set -v
#FAKE=echo


parse_xipinit() {
    #looks for xipinit.sh script, and parses its settings
    # save parameters from environment
    _RODEV="$rodev"
    _RODIRS="$rodirs"
    _XIPIMAGE="$xipimage"
    _ROMOUNT="$romount"
    # now parse the script
    echo "reading" /sbin/xipinit.sh
    eval $(cat /sbin/xipinit.sh | grep "RODIRS=" |grep -v 'RODIRS=\"\$' |grep -v "#RODIRS=")
    eval $(cat /sbin/xipinit.sh | grep "RODEV=" |grep -v "#RODEV=")
    eval $(cat /sbin/xipinit.sh | grep "XIPIMAGE=" |grep -v "#XIPIMAGE=")
    eval $(cat /sbin/xipinit.sh | grep "ROMOUNT=" |grep -v "#ROMOUNT=")
    # override parameters with saved environment
    RODEV="${_RODEV:-$RODEV}"
    RODIRS=${_RODIRS:-$RODIRS}
    XIPIMAGE=${_XIPIMAGE:-$XIPIMAGE}
    ROMOUNT=${_ROMOUNT:-$ROMOUNT}
    # make sure it ends with ,
    RODIRS="$RODIRS",
}


check_blockdev() {
    #load dcss block device driver if needed
    test -d /sys/devices/dcssblk || ( echo "loading dcss block device driver" && modprobe dcssblk )
    if [ ! -d /sys/devices/dcssblk ]; then
	echo "error: could not initialize dcss block device driver"
	exit 1
    fi
    echo "dcss block device driver found"
}

load_segment() {
    #load the segment using the block device
    echo "loading segment" $XIPIMAGE
    $FAKE echo $XIPIMAGE >/sys/devices/dcssblk/add
    if [ ! -d /sys/devices/dcssblk/$XIPIMAGE ]; then
	echo "error: failed to load segment"
	exit 1
    fi
    echo "reloading segment" $XIPIMAGE "in nonshared mode"
    $FAKE echo 0 >/sys/devices/dcssblk/$XIPIMAGE/shared
    TEMPVAR0=$(cat /sys/devices/dcssblk/$XIPIMAGE/shared)
    if [ $TEMPVAR0 -ne 0 ]; then
	echo "error: cannot change segment to nonshared mode, maybe you don't have class E privileges?"
	exit 1
    fi
}

unload_segment() {
    #unload the segment again
    echo "unloading segment" $XIPIMAGE
    $FAKE echo $XIPIMAGE >/sys/devices/dcssblk/remove
    if [ -d /sys/devices/dcssblk/$XIPIMAGE ]; then
	echo "error: failed to unload segment. is it still busy?"
	exit 1
    fi
}

create_devnode() {
    #create a temporary device node for mounting
    if [ -e $TEMPNODE ]; then
	echo "error: temporary device node" $TEMPNODE "already exists."
	echo "if it still exists from a former run, please delete it and try again"
	exit 1
    fi
    if [ ! -f /sys/devices/dcssblk/$XIPIMAGE/block/dev ]; then
	echo "unexpected-error: cannot find block device directory in sysfs, please report to linux390@de.ibm.com"
	exit 1
    fi
    MAJOR=$(cat /sys/devices/dcssblk/$XIPIMAGE/block/dev |sed -e "s/\:[0-9]\{1,\}//g")
    MINOR=$(cat /sys/devices/dcssblk/$XIPIMAGE/block/dev |sed -e "s/[0-9]\{1,\}\://g")
    mknod $TEMPNODE b $MAJOR $MINOR
    if [ ! -b $TEMPNODE ]; then
	echo "error: cannot not create temporary device node" $TEMPNODE
        exit 1
    fi
}

delete_devnode() {
    #delete temporary device node again
    rm -f $TEMPNODE
    if [ -e $TEMPNODE ]; then
	echo "unexpected-error: cannot remove temporary device node" $TEMPNODE ", please report to linux390@de.ibm.com"
	exit 1
    fi
}

mount_segment() {
    #this mounts the segment on $ROMOUNT
    echo "mounting segment" $XIPIMAGE "on" $ROMOUNT
    $FAKE mount -t ext2 $TEMPNODE $ROMOUNT
    TEMPVAR0=$(cat /proc/mounts |grep $TEMPNODE| wc -l)
    if [ ! 1 -eq $TEMPVAR0 ]; then
	echo "error: cannot mount segment, is either already mounted or operation failed"
	umount $TEMPNODE
	delete_devnode
	unload_segment
	exit 1;
    fi
}

unmount_segment() {
    #unmounts the segment from $ROMOUNT
    echo "unmounting segment" $XIPIMAGE
    $FAKE umount $TEMPNODE
}

update_segment() {
    #copies all data from disk to segment
    echo "updating software on segment"
    echo "phase 1: deleting old content"
    TEMPVAR0=$(echo $RODIRS)
    while [ -n "$TEMPVAR0" ]; do
	dir="${TEMPVAR0%%,*}"
	TEMPVAR0="${TEMPVAR0#*,}"
	if [ ! -n "$TEMPVAR0" ]; then
	    continue
	fi
	echo "deleting directory" $ROMOUNT/$dir
	if [ ! -d "$ROMOUNT/$dir" ]; then
	    echo "error:cannot find directory" $ROMOUNT/$dir
	    echo "does not exist on dcss or is not a directory"
            unmount_segment
	    delete_devnode
	    unload_segment
	    echo "cannot recover from earlier error - exit"
	    exit 1
	fi
	$FAKE rm -rf $ROMOUNT/$dir
	if [ -d "$ROMOUNT/$dir" ]; then
	    echo "error:cannot delete directory" $ROMOUNT/$dir
            unmount_segment
	    delete_devnode
	    unload_segment
	    echo "cannot recover from earlier error - exit"
	    exit 1
	fi
    done
    echo "phase 2: copy new content"
    TEMPVAR0=$(echo $RODIRS)
    while [ -n "$TEMPVAR0" ]; do
	dir="${TEMPVAR0%%,*}"
	TEMPVAR0="${TEMPVAR0#*,}"
	if [ ! -n "$TEMPVAR0" ]; then
	    continue
	fi
	echo "copying directory" $ROMOUNT/$dir
	mkdir -p $ROMOUNT/$dir
	if [ ! -d "$ROMOUNT/$dir" ]; then
	    echo "error:cannot create directory" $ROMOUNT/$dir
            unmount_segment
	    delete_devnode
	    unload_segment
	    echo "cannot recover from earlier error - exit"
	    exit 1
	fi
	cp -a /$dir/* $ROMOUNT/$dir
	if [ $? -ne 0 ]; then
	    echo "error:cannot copy directory" $ROMOUNT/$dir
            unmount_segment
	    delete_devnode
	    unload_segment
	    echo "cannot recover from earlier error - exit"
	    exit 1
 	fi
    done
}

save_update () {
    if [ ! -f /sys/devices/dcssblk/$XIPIMAGE/save ]; then
	echo "error:segment or block device driver not loaded"
	exit 1
    fi

    echo 1 >/sys/devices/dcssblk/$XIPIMAGE/save
    TEMPVAR0=$(cat /sys/devices/dcssblk/$XIPIMAGE/save)
    if [ $TEMPVAR0 -eq 0 ]; then
	echo "success: segment saved"
    fi
    if [ $TEMPVAR0 -eq 1 ]; then
	echo "warning: segment scheduled to be saved when it becomes idle, check why it is busy!"
    fi
}


parse_xipinit
check_blockdev
load_segment
create_devnode
mount_segment
update_segment
unmount_segment
delete_devnode
save_update
unload_segment
